@@ -11,16 +11,16 @@ from pygolf.unparser import Unparser
 class Pygolfer:
     def shorten(self, code: str) -> str:
         module: NodeNG = ast.parse(code)
-        unparser: Unparser = Unparser()
+        unparser: Unparser = Unparser(should_remove_spaces_between_keywords=False)
         rules: List[AstroidRule] = []
 
         for phase in all_phases:
             for rule in phase.generate_rules(module):
                 ast.MANAGER.register_transform(rule.on_node, rule.transform, rule.predicate)
                 rules.append(rule)
-
             module = ast.parse(unparser.unparse(module))
 
         for rule in rules:
             ast.MANAGER.unregister_transform(rule.on_node, rule.transform, rule.predicate)
+        unparser.should_remove_spaces_between_keywords = True
         return unparser.unparse(module)
