@@ -7,8 +7,9 @@ from pygolf.errors.python_2_code_detected import Python2CodeDetected
 
 
 class Unparser:
-    def __init__(self) -> None:
+    def __init__(self, should_remove_spaces_between_keywords: bool = True) -> None:
         self.sep: str = " "
+        self.should_remove_spaces_between_keywords = should_remove_spaces_between_keywords
 
     def unparse(self, node: NodeNG, indent: int = 0) -> str:
         method = getattr(self, "unparse_" + node.__class__.__name__)
@@ -39,8 +40,8 @@ class Unparser:
     def unparse_for(self, for_node: Union[ast.AsyncFor, ast.For], is_async: bool, indent: int = 0) -> str:
         for_stmnt = (
             f"{self.sep*indent}{'async ' if is_async else ''}"
-            + f"for {self.unparse(for_node.target)} in{self.space_after_kw(for_node.iter)}"
-            + f"{self.unparse(for_node.iter)}:{self.unparse_block(for_node.body, indent)}"
+            f"for {self.unparse(for_node.target)} in{self.space_after_kw(for_node.iter)}"
+            f"{self.unparse(for_node.iter)}:{self.unparse_block(for_node.body, indent)}"
         )
 
         if for_node.orelse:
@@ -65,8 +66,7 @@ class Unparser:
     def unparse_with(self, node: Union[ast.AsyncWith, ast.With], is_async: bool, indent: int = 0) -> str:
         return (
             f"{self.sep*indent}{'async ' if is_async else ''}with "
-            + f"{','.join(map(self.unparse_with_item, node.items))}:"
-            + self.unparse_block(node.body, indent)
+            f"{','.join(map(self.unparse_with_item, node.items))}:" + self.unparse_block(node.body, indent)
         )
 
     def unparse_with_item(self, node: List[Tuple[NodeNG, Optional[ast.AssignName]]]) -> str:
@@ -76,7 +76,10 @@ class Unparser:
         return f"{self.unparse(item)} as {self.unparse(alias)}"
 
     def unparse_AnnAssign(self, node: ast.AnnAssign, indent: int = 0) -> str:
-        return f"{self.sep*indent}{node.target.name}={self.unparse(node.value)}"  # ignore annotation
+        if node.value is not None:
+            name = node.target.name if hasattr(node.target, "name") else node.target.attrname
+            return f"{self.sep*indent}{name}={self.unparse(node.value)}"  # ignore annotation
+        return f"{self.sep*indent}{node.target.name}"
 
     def unparse_Arguments(self, node: ast.Arguments, indent: int = 0) -> str:
         number_non_default_args = len(node.args) - len(node.defaults)
@@ -143,8 +146,12 @@ class Unparser:
 
         stmnt = ""
 
-        def unparse_child(node: ast.BinOp, child_node: NodeNG) -> str:
-            if isinstance(child_node, ast.BinOp) and operators_level[node.op] > operators_level[child_node.op]:
+        def unparse_child(node: ast.BinOp, child_node: NodeNG, already_have_parenthesis=False) -> str:
+            if (
+                not already_have_parenthesis
+                and isinstance(child_node, ast.BinOp)
+                and operators_level[node.op] > operators_level[child_node.op]
+            ):
                 return f"({self.unparse(child_node)})"
             return f"{self.unparse(child_node)}"
 
@@ -157,12 +164,12 @@ class Unparser:
                 or (
                     not isinstance(node.left, ast.BinOp) and not isinstance(node.left, ast.Const)
                 )  # Possibly a string is returned, in doubt we have to put parenthesis
+                or (isinstance(node.right, ast.Compare))
             )
-            and node.op == "*"
-            and isinstance(node.right, ast.BinOp)
-            and node.right.op in ("*", "//", "%")
+            and operators_level[node.op] > 4
+            and (isinstance(node.right, ast.BinOp) or isinstance(node.right, ast.Compare))
         ):
-            stmnt += f"({unparse_child(node, node.right)})"
+            stmnt += f"({unparse_child(node, node.right, already_have_parenthesis=True)})"
         else:
             stmnt += unparse_child(node, node.right)
 
@@ -306,7 +313,7 @@ class Unparser:
     def unparse_GeneratorExp(self, node: ast.GeneratorExp, indent: int = 0) -> str:
         return (
             f"{self.sep*indent}({self.unparse(node.elt)}{self.space_before_kw(node.elt)}"
-            + f"{self.unparse_comprehension_generators(node.generators)})"
+            f"{self.unparse_comprehension_generators(node.generators)})"
         )
 
     def unparse_Global(self, node: ast.Global, indent: int = 0) -> str:
@@ -315,7 +322,7 @@ class Unparser:
     def unparse_If(self, node: ast.If, indent: int = 0) -> str:
         stmnt = (
             f"{self.sep*indent}if{self.space_after_kw(node.test)}{self.unparse(node.test)}:"
-            + f"{self.unparse_block(node.body, indent)}"
+            f"{self.unparse_block(node.body, indent)}"
         )
         if node.orelse:
             stmnt += f"\n{self.sep*indent}else:{self.unparse_block(node.orelse, indent)}"
@@ -324,9 +331,9 @@ class Unparser:
     def unparse_IfExp(self, node: ast.IfExp, indent: int = 0) -> str:
         return (
             f"{self.sep*indent}{self.unparse(node.body)}{self.space_before_kw(node.body)}"
-            + f"if{self.space_after_kw(node.test)}{self.unparse(node.test)}"
-            + f"{self.space_after_kw(node.test)}else"
-            + f"{self.space_after_kw(node.orelse)}{self.unparse(node.orelse)}"
+            f"if{self.space_after_kw(node.test)}{self.unparse(node.test)}"
+            f"{self.space_after_kw(node.test)}else"
+            f"{self.space_after_kw(node.orelse)}{self.unparse(node.orelse)}"
         )
 
     def unparse_Import(self, node: ast.Import, indent: int = 0) -> str:
@@ -335,8 +342,8 @@ class Unparser:
     def unparse_ImportFrom(self, node: ast.ImportFrom, indent: int = 0) -> str:
         return (
             f"{self.sep*indent}from "
-            + f"{'.'*node.level if node.level is not None else ''}{node.modname} "
-            + f"import {','.join(map(self.unparse_alias_import, node.names))}"
+            f"{'.'*node.level if node.level is not None else ''}{node.modname} "
+            f"import {','.join(map(self.unparse_alias_import, node.names))}"
         )
 
     def unparse_Index(self, node: ast.Index, indent: int = 0) -> str:
@@ -359,7 +366,7 @@ class Unparser:
     def unparse_ListComp(self, node: ast.ListComp, indent: int = 0) -> str:
         return (
             f"{self.sep*indent}[{self.unparse(node.elt)}{self.space_before_kw(node.elt)}"
-            + f"{self.unparse_comprehension_generators(node.generators)}]"
+            f"{self.unparse_comprehension_generators(node.generators)}]"
         )
 
     def unparse_Module(self, node: ast.Module, indent: int = 0) -> str:
@@ -404,7 +411,7 @@ class Unparser:
     def unparse_SetComp(self, node: ast.SetComp, indent: int = 0) -> str:
         return (
             f"{self.sep*indent}{{{self.unparse(node.elt)}{self.space_before_kw(node.elt)}"
-            + f"{self.unparse_comprehension_generators(node.generators)}}}"
+            f"{self.unparse_comprehension_generators(node.generators)}}}"
         )
 
     def unparse_Slice(self, node: ast.Slice, indent: int = 0) -> str:
@@ -447,7 +454,11 @@ class Unparser:
         return self.sep * indent + core
 
     def unparse_UnaryOp(self, node: ast.UnaryOp, indent: int = 0) -> str:
-        return f"{self.sep*indent}{node.op}{self.unparse(node.operand)}"
+        return (
+            f"{self.sep*indent}{node.op}"
+            f"{' ' if node.op == 'not' and not self._can_follow_reserved_keywords(node.operand) else ''}"
+            f"{self.unparse(node.operand)}"
+        )
 
     def unparse_Unknown(self, node: ast.Unknown, indent: int = 0) -> str:
         pass
@@ -476,10 +487,14 @@ class Unparser:
         return "" if self._can_be_before_reserved_keywords(node) else " "
 
     def _can_follow_reserved_keywords(self, node):
+        if not self.should_remove_spaces_between_keywords:
+            return False
         unparsed = self.unparse(node)
         return isinstance(unparsed, str) and unparsed[0] in "'\"({["
 
     def _can_be_before_reserved_keywords(self, node):
+        if not self.should_remove_spaces_between_keywords:
+            return False
         unparsed = self.unparse(node)
         return isinstance(unparsed, str) and unparsed[-1] in "'\")}]0123456789"
 
